@@ -53,11 +53,12 @@ const wchar_t *contrib_core_strs[] = {
 	L"Ilya Zhuravlev (thexyz) <xyz@minetest.net>",
 	L"Lisa Milne (darkrose) <lisa@ltmnet.com>",
 	L"Maciej Kasatkin (RealBadAngel) <mk@realbadangel.pl>",
-	L"proller <proler@gmail.com>"
+	L"proller <proler@gmail.com>",
+	L"sfan5 <sfan5@live.de>",
+	L"kahrl <kahrl@gmx.net>"
 };
 
 const wchar_t *contrib_active_strs[] = {
-	L"sfan5 <sfan5@live.de>",
 	L"sapier <sapier@gmx.net>",
 	L"Vanessa Ezekowitz (VanessaE) <vanessaezekowitz@gmail.com>",
 	L"Jurgen Doser (doserj) <jurgen.doser@gmail.com>",
@@ -69,7 +70,6 @@ const wchar_t *contrib_active_strs[] = {
 };
 
 const wchar_t *contrib_previous_strs[] = {
-	L"kahrl <kahrl@gmx.net>",
 	L"Giuseppe Bilotta (Oblomov) <giuseppe.bilotta@gmail.com>",
 	L"Jonathan Neuschafer <j.neuschaefer@gmx.net>",
 	L"Nils Dagsson Moskopp (erlehmann) <nils@dieweltistgarnichtso.net>",
@@ -89,7 +89,9 @@ struct CreateWorldDestMainMenu : public CreateWorldDest
 		std::string name_narrow = wide_to_narrow(name);
 		if(!string_allowed_blacklist(name_narrow, WORLDNAME_BLACKLISTED_CHARS))
 		{
-			m_menu->displayMessageMenu(wgettext("Cannot create world: Name contains invalid characters"));
+			wchar_t* text = wgettext("Cannot create world: Name contains invalid characters");
+			m_menu->displayMessageMenu(text);
+			delete[] text;
 			return;
 		}
 		std::vector<WorldSpec> worlds = getAvailableWorlds();
@@ -98,7 +100,9 @@ struct CreateWorldDestMainMenu : public CreateWorldDest
 		{
 			if((*i).name == name_narrow)
 			{
-				m_menu->displayMessageMenu(wgettext("Cannot create world: A world by this name already exists"));
+				wchar_t* text = wgettext("Cannot create world: A world by this name already exists");
+				m_menu->displayMessageMenu(text);
+				delete[] text;
 				return;
 			}
 		}
@@ -158,15 +162,8 @@ enum
 	GUI_ID_SERVERLIST_TOGGLE,
 	GUI_ID_SERVERLIST_DELETE,
 	GUI_ID_SERVERLIST_TITLE,
-};
-
-enum
-{
-	TAB_SINGLEPLAYER=0,
-	TAB_MULTIPLAYER,
-	TAB_ADVANCED,
-	TAB_SETTINGS,
-	TAB_CREDITS
+	GUI_ID_GAME_BUTTON_FIRST = 130,
+	GUI_ID_GAME_BUTTON_MAX = 150,
 };
 
 GUIMainMenu::GUIMainMenu(gui::IGUIEnvironment* env,
@@ -251,8 +248,13 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 	{
 		core::rect<s32> rect(0, 0, size.X, 40);
 		rect += v2s32(4, 0);
-		Environment->addStaticText(narrow_to_wide(
-				"Minetest " VERSION_STRING).c_str(),
+		std::string t = "Minetest " VERSION_STRING;
+		if(m_data->selected_game_name != "" &&
+				m_data->selected_tab == TAB_SINGLEPLAYER){
+			t += "/";
+			t += m_data->selected_game_name;
+		}
+		Environment->addStaticText(narrow_to_wide(t).c_str(),
 				rect, false, true, this, -1);
 	}
 
@@ -280,12 +282,24 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 		rect += m_topleft_client + v2s32(0, -30);
 		gui::IGUITabControl *e = Environment->addTabControl(
 				rect, this, true, true, GUI_ID_TAB_CONTROL);
-		e->addTab(wgettext("Singleplayer"));
-		e->addTab(wgettext("Multiplayer"));
-		e->addTab(wgettext("Advanced"));
-		e->addTab(wgettext("Settings"));
-		e->addTab(wgettext("Credits"));
+		wchar_t* text = wgettext("Singleplayer");
+		e->addTab(text);
+		delete[] text;
+		text = wgettext("Multiplayer");
+		e->addTab(text);
+		delete[] text;
+		text = wgettext("Advanced");
+		e->addTab(text);
+		delete[] text;
+		text = wgettext("Settings");
+		e->addTab(text);
+		delete[] text;
+		text = wgettext("Credits");
+		e->addTab(text);
+		delete[] text;
+
 		e->setActiveTab(m_data->selected_tab);
+
 	}
 	
 	if(m_data->selected_tab == TAB_SINGLEPLAYER)
@@ -313,9 +327,11 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 		{
 			core::rect<s32> rect(0, 0, world_sel_w-4, 20);
 			rect += m_topleft_client + v2s32(world_sel_x+4, world_sel_y-20);
+			wchar_t* text = wgettext("Select World:");
 			/*gui::IGUIStaticText *e =*/ Environment->addStaticText(
-					wgettext("Select World:"), 
+					text,
 					rect, false, true, this, -1);
+			delete[] text;
 			/*e->setTextAlignment(gui::EGUIA_CENTER, gui::EGUIA_CENTER);*/
 		}
 		{
@@ -324,34 +340,46 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 			gui::IGUIListBox *e = Environment->addListBox(rect, this,
 					GUI_ID_WORLD_LISTBOX);
 			e->setDrawBackground(true);
-			for(std::vector<WorldSpec>::const_iterator i = m_data->worlds.begin();
-					i != m_data->worlds.end(); i++){
-				e->addItem(narrow_to_wide(i->name+" ["+i->gameid+"]").c_str());
+			m_world_indices.clear();
+			for(size_t wi = 0; wi < m_data->worlds.size(); wi++){
+				const WorldSpec &spec = m_data->worlds[wi];
+				if(spec.gameid == m_data->selected_game){
+					//e->addItem(narrow_to_wide(spec.name+" ["+spec.gameid+"]").c_str());
+					e->addItem(narrow_to_wide(spec.name).c_str());
+					m_world_indices.push_back(wi);
+					if(m_data->selected_world == (int)wi)
+						e->setSelected(m_world_indices.size()-1);
+				}
 			}
-			e->setSelected(m_data->selected_world);
 			Environment->setFocus(e);
 		}
 		// Delete world button
 		{
 			core::rect<s32> rect(0, 0, world_button_w, 30);
 			rect += m_topleft_client + v2s32(world_sel_x, world_sel_y+world_sel_h+0);
+			wchar_t* text = wgettext("Delete");
 			Environment->addButton(rect, this, GUI_ID_DELETE_WORLD_BUTTON,
-				  wgettext("Delete"));
+				 text);
+			delete[] text;
 		}
 		// Create world button
 		{
 			core::rect<s32> rect(0, 0, world_button_w, 30);
 			rect += m_topleft_client + v2s32(world_sel_x+world_button_w+bs, world_sel_y+world_sel_h+0);
+			wchar_t* text = wgettext("New");
 			Environment->addButton(rect, this, GUI_ID_CREATE_WORLD_BUTTON,
-				  wgettext("New"));
+				 text);
+			delete[] text;
 		}
 		// Configure world button
 		{
 			core::rect<s32> rect(0, 0, world_button_w, 30);
 			rect += m_topleft_client + v2s32(world_sel_x+(world_button_w+bs)*2,
 					world_sel_y+world_sel_h+0);
+			wchar_t* text = wgettext("Configure");
 			Environment->addButton(rect, this, GUI_ID_CONFIGURE_WORLD_BUTTON,
-				  wgettext("Configure"));
+				text);
+			delete[] text;
 		}
 		// Start game button
 		{
@@ -365,8 +393,10 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 			core::rect<s32> rect(0, 0, bw, 30);
 			rect += m_topleft_client + v2s32(world_sel_x+world_sel_w-bw,
 					world_sel_y+world_sel_h+30+bs);
+			wchar_t* text = wgettext("Play");
 			Environment->addButton(rect, this,
-					GUI_ID_JOIN_GAME_BUTTON, wgettext("Play"));
+					GUI_ID_JOIN_GAME_BUTTON, text);
+			delete[] text;
 		}
 		// Options
 		s32 option_x = 50;
@@ -376,16 +406,40 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 		{
 			core::rect<s32> rect(0, 0, option_w, 30);
 			rect += m_topleft_client + v2s32(option_x, option_y+20*0);
+			wchar_t* text = wgettext("Creative Mode");
 			Environment->addCheckBox(m_data->creative_mode, rect, this,
-					GUI_ID_CREATIVE_CB, wgettext("Creative Mode"));
+					GUI_ID_CREATIVE_CB, text);
+			delete[] text;
 		}
 		{
 			core::rect<s32> rect(0, 0, option_w, 30);
 			rect += m_topleft_client + v2s32(option_x, option_y+20*1);
+			wchar_t* text = wgettext("Enable Damage");
 			Environment->addCheckBox(m_data->enable_damage, rect, this,
-					GUI_ID_DAMAGE_CB, wgettext("Enable Damage"));
+					GUI_ID_DAMAGE_CB, text);
+			delete[] text;
 		}
 		changeCtype("C");
+
+		/* Add game selection buttons */
+		video::IVideoDriver* driver = Environment->getVideoDriver();
+		for(size_t i=0; i<m_data->games.size(); i++){
+			const SubgameSpec *spec = &m_data->games[i];
+			v2s32 p(8 + i*(48+8), screensize.Y - (48+8));
+			core::rect<s32> rect(0, 0, 48, 48);
+			rect += p;
+			video::ITexture *bgtexture = NULL;
+			if(spec->menuicon_path != "")
+				bgtexture = driver->getTexture(spec->menuicon_path.c_str());
+			gui::IGUIButton *b = Environment->addButton(rect, this,
+					GUI_ID_GAME_BUTTON_FIRST+i, narrow_to_wide(wrap_rows(spec->id, 4)).c_str());
+			if(bgtexture){
+				b->setImage(bgtexture);
+				b->setText(L"");
+				b->setDrawBorder(false);
+				b->setUseAlphaChannel(true);
+			}
+		}
 	}
 	else if(m_data->selected_tab == TAB_MULTIPLAYER)
 	{
@@ -402,9 +456,11 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 		// Nickname + password
 		{
 			core::rect<s32> rect(0, 0, 110, 20);
+			wchar_t* text = wgettext("Name/Password");
 			rect += m_topleft_client + v2s32(m_size_client.X-60-100, 10+6);
-			Environment->addStaticText(wgettext("Name/Password"), 
+			Environment->addStaticText(text,
 				rect, false, true, this, -1);
+			delete [] text;
 		}
 		changeCtype("C");
 		{
@@ -439,16 +495,20 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 				{
 					core::rect<s32> rect(0, 0, 390, 20);
 					rect += m_topleft_client + v2s32(50, 10);
-					Environment->addStaticText(wgettext("Favorites:"),
+					wchar_t* text = wgettext("Favorites:");
+					Environment->addStaticText(text,
 						rect, false, true, this, GUI_ID_SERVERLIST_TITLE);
+					delete[] text;
 				}
 			} else {
 				m_data->servers = ServerList::getOnline();
 				{
 					core::rect<s32> rect(0, 0, 390, 20);
 					rect += m_topleft_client + v2s32(50, 10);
-					Environment->addStaticText(wgettext("Public Server List:"),
+					wchar_t* text = wgettext("Public Server List:");
+					Environment->addStaticText(text,
 						rect, false, true, this, GUI_ID_SERVERLIST_TITLE);
+					delete[] text;
 				}
 			}
 #else
@@ -456,8 +516,10 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 			{
 				core::rect<s32> rect(0, 0, 390, 20);
 				rect += m_topleft_client + v2s32(50, 10);
-				Environment->addStaticText(wgettext("Favorites:"),
+				wchar_t* text = wgettext("Favorites:");
+				Environment->addStaticText(text,
 					rect, false, true, this, GUI_ID_SERVERLIST_TITLE);
+				delete[] text;
 			}
 #endif
 			updateGuiServerList();
@@ -467,8 +529,10 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 		{
 			core::rect<s32> rect(0, 0, 110, 20);
 			rect += m_topleft_client + v2s32(50, m_size_client.Y-50-15+6);
-			Environment->addStaticText(wgettext("Address/Port"),
+			wchar_t* text = wgettext("Address/Port");
+			Environment->addStaticText(text,
 				rect, false, true, this, -1);
+			delete [] text;
 		}
 		changeCtype("C");
 		{
@@ -493,13 +557,17 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 			core::rect<s32> rect(0, 0, 260, 30);
 			rect += m_topleft_client + v2s32(50,
 					180);
+			wchar_t* text = wgettext("Show Public");
 			gui::IGUIButton *e = Environment->addButton(rect, this, GUI_ID_SERVERLIST_TOGGLE,
-				wgettext("Show Public"));
+				text);
+			delete[] text;
 			e->setIsPushButton(true);
 			if (m_data->selected_serverlist == SERVERLIST_PUBLIC)
 			{
-				e->setText(wgettext("Show Favorites"));
+				wchar_t* text = wgettext("Show Favorites");
+				e->setText(text);
 				e->setPressed();
+				delete[] text;
 			}
 		}
 		#endif
@@ -507,18 +575,23 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 		{
 			core::rect<s32> rect(0, 0, 120, 30);
 			rect += m_topleft_client + v2s32(50+260+10, 180);
+			wchar_t* text = wgettext("Delete");
 			gui::IGUIButton *e = Environment->addButton(rect, this, GUI_ID_SERVERLIST_DELETE,
-				wgettext("Delete"));
+					text);
 			if (m_data->selected_serverlist == SERVERLIST_PUBLIC) // Hidden when on public list
 				e->setVisible(false);
+
+			delete [] text;
 		}
 		// Start game button
 		{
 			core::rect<s32> rect(0, 0, 120, 30);
 			rect += m_topleft_client + v2s32(m_size_client.X-130-30,
 					m_size_client.Y-25-15);
+			wchar_t* text = wgettext("Connect");
 			Environment->addButton(rect, this, GUI_ID_JOIN_GAME_BUTTON,
-				wgettext("Connect"));
+				text);
+			delete[] text;
 		}
 		changeCtype("C");
 	}
@@ -538,8 +611,10 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 		{
 			core::rect<s32> rect(0, 0, 110, 20);
 			rect += m_topleft_client + v2s32(35+30, 35+6);
-			Environment->addStaticText(wgettext("Name/Password"), 
+			wchar_t* text = wgettext("Name/Password");
+			Environment->addStaticText(text,
 				rect, false, true, this, -1);
+			delete [] text;
 		}
 		changeCtype("C");
 		{
@@ -565,8 +640,10 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 		{
 			core::rect<s32> rect(0, 0, 110, 20);
 			rect += m_topleft_client + v2s32(35+30, 75+6);
-			Environment->addStaticText(wgettext("Address/Port"),
+			wchar_t* text = wgettext("Address/Port");
+			Environment->addStaticText(text,
 				rect, false, true, this, -1);
+			delete[] text;
 		}
 		changeCtype("C");
 		{
@@ -588,16 +665,20 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 		{
 			core::rect<s32> rect(0, 0, 400, 20);
 			rect += m_topleft_client + v2s32(160+30, 75+35);
-			Environment->addStaticText(wgettext("Leave address blank to start a local server."),
+			wchar_t* text = wgettext("Leave address blank to start a local server.");
+			Environment->addStaticText(text,
 				rect, false, true, this, -1);
+			delete[] text;
 		}
 		// Start game button
 		{
 			core::rect<s32> rect(0, 0, 180, 30);
 			rect += m_topleft_client + v2s32(m_size_client.X-180-30,
 					m_size_client.Y-30-20);
+			wchar_t* text = wgettext("Start Game / Connect");
 			Environment->addButton(rect, this, GUI_ID_JOIN_GAME_BUTTON,
-				wgettext("Start Game / Connect"));
+				text);
+			delete[] text;
 		}
 		/*
 			Server section
@@ -615,36 +696,46 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 		{
 			core::rect<s32> rect(0, 0, 250, 30);
 			rect += m_topleft_server + v2s32(30+20+250+20, 20);
+			wchar_t* text = wgettext("Creative Mode");
 			Environment->addCheckBox(m_data->creative_mode, rect, this, GUI_ID_CREATIVE_CB,
-				wgettext("Creative Mode"));
+				text);
+			delete[] text;
 		}
 		{
 			core::rect<s32> rect(0, 0, 250, 30);
 			rect += m_topleft_server + v2s32(30+20+250+20, 40);
+			wchar_t* text = wgettext("Enable Damage");
 			Environment->addCheckBox(m_data->enable_damage, rect, this, GUI_ID_DAMAGE_CB,
-				wgettext("Enable Damage"));
+				text);
+			delete[] text;
 		}
 		#if USE_CURL
 		{
 			core::rect<s32> rect(0, 0, 250, 30);
 			rect += m_topleft_server + v2s32(30+20+250+20, 60);
+			wchar_t* text = wgettext("Public");
 			Environment->addCheckBox(m_data->enable_public, rect, this, GUI_ID_PUBLIC_CB,
-				wgettext("Public"));
+				text);
+			delete[] text;
 		}
 		#endif
 		// Delete world button
 		{
 			core::rect<s32> rect(0, 0, 130, 30);
 			rect += m_topleft_server + v2s32(30+20+250+20, 90);
+			wchar_t* text = wgettext("Delete world");
 			Environment->addButton(rect, this, GUI_ID_DELETE_WORLD_BUTTON,
-				  wgettext("Delete world"));
+				 text );
+			delete[] text;
 		}
 		// Create world button
 		{
 			core::rect<s32> rect(0, 0, 130, 30);
 			rect += m_topleft_server + v2s32(30+20+250+20+140, 90);
+			wchar_t* text = wgettext("Create world");
 			Environment->addButton(rect, this, GUI_ID_CREATE_WORLD_BUTTON,
-				  wgettext("Create world"));
+				 text );
+			delete[] text;
 		}
 		// World selection listbox
 		{
@@ -653,9 +744,11 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 			gui::IGUIListBox *e = Environment->addListBox(rect, this,
 					GUI_ID_WORLD_LISTBOX);
 			e->setDrawBackground(true);
-			for(std::vector<WorldSpec>::const_iterator i = m_data->worlds.begin();
-					i != m_data->worlds.end(); i++){
-				e->addItem(narrow_to_wide(i->name+" ["+i->gameid+"]").c_str());
+			m_world_indices.clear();
+			for(size_t wi = 0; wi < m_data->worlds.size(); wi++){
+				const WorldSpec &spec = m_data->worlds[wi];
+				e->addItem(narrow_to_wide(spec.name+" ["+spec.gameid+"]").c_str());
+				m_world_indices.push_back(wi);
 			}
 			e->setSelected(m_data->selected_world);
 		}
@@ -677,26 +770,34 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 		{
 			core::rect<s32> rect(0, 0, option_w, 30);
 			rect += m_topleft_client + v2s32(option_x, option_y);
+			wchar_t* text = wgettext("Fancy trees");
 			Environment->addCheckBox(m_data->fancy_trees, rect, this,
-					GUI_ID_FANCYTREE_CB, wgettext("Fancy trees")); 
+					GUI_ID_FANCYTREE_CB, text);
+			delete[] text;
 		}
 		{
 			core::rect<s32> rect(0, 0, option_w, 30);
 			rect += m_topleft_client + v2s32(option_x, option_y+20);
+			wchar_t* text = wgettext("Smooth Lighting");
 			Environment->addCheckBox(m_data->smooth_lighting, rect, this,
-					GUI_ID_SMOOTH_LIGHTING_CB, wgettext("Smooth Lighting"));
+					GUI_ID_SMOOTH_LIGHTING_CB, text);
+			delete[] text;
 		}
 		{
 			core::rect<s32> rect(0, 0, option_w, 30);
 			rect += m_topleft_client + v2s32(option_x, option_y+20*2);
+			wchar_t* text = wgettext("3D Clouds");
 			Environment->addCheckBox(m_data->clouds_3d, rect, this,
-					GUI_ID_3D_CLOUDS_CB, wgettext("3D Clouds"));
+					GUI_ID_3D_CLOUDS_CB, text);
+			delete[] text;
 		}
 		{
 			core::rect<s32> rect(0, 0, option_w, 30);
 			rect += m_topleft_client + v2s32(option_x, option_y+20*3);
+			wchar_t* text = wgettext("Opaque water");
 			Environment->addCheckBox(m_data->opaque_water, rect, this,
-					GUI_ID_OPAQUE_WATER_CB, wgettext("Opaque water"));
+					GUI_ID_OPAQUE_WATER_CB, text);
+			delete[] text;
 		}
 
 
@@ -705,58 +806,74 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 		{
 			core::rect<s32> rect(0, 0, option_w+20, 30);
 			rect += m_topleft_client + v2s32(option_x+175, option_y);
+			wchar_t* text = wgettext("Mip-Mapping");
 			Environment->addCheckBox(m_data->mip_map, rect, this,
-				       GUI_ID_MIPMAP_CB, wgettext("Mip-Mapping"));
+				       GUI_ID_MIPMAP_CB, text);
+			delete[] text;
 		}
 
 		{
 			core::rect<s32> rect(0, 0, option_w+20, 30);
 			rect += m_topleft_client + v2s32(option_x+175, option_y+20);
+			wchar_t* text = wgettext("Anisotropic Filtering");
 			Environment->addCheckBox(m_data->anisotropic_filter, rect, this,
-				       GUI_ID_ANISOTROPIC_CB, wgettext("Anisotropic Filtering"));
+				       GUI_ID_ANISOTROPIC_CB, text);
+			delete[] text;
 		}
 
 		{
 			core::rect<s32> rect(0, 0, option_w+20, 30);
 			rect += m_topleft_client + v2s32(option_x+175, option_y+20*2);
+			wchar_t* text = wgettext("Bi-Linear Filtering");
 			Environment->addCheckBox(m_data->bilinear_filter, rect, this,
-				       GUI_ID_BILINEAR_CB, wgettext("Bi-Linear Filtering"));
+				       GUI_ID_BILINEAR_CB, text);
+			delete[] text;
 		}
 
 		{
 			core::rect<s32> rect(0, 0, option_w+20, 30);
 			rect += m_topleft_client + v2s32(option_x+175, option_y+20*3);
+			wchar_t* text = wgettext("Tri-Linear Filtering");
 			Environment->addCheckBox(m_data->trilinear_filter, rect, this,
-				       GUI_ID_TRILINEAR_CB, wgettext("Tri-Linear Filtering"));
+				       GUI_ID_TRILINEAR_CB, text);
+			delete[] text;
 		}
 
 		// shader/on demand image loading/particles settings
 		{
 			core::rect<s32> rect(0, 0, option_w+20, 30);
 			rect += m_topleft_client + v2s32(option_x+175*2, option_y);
+			wchar_t* text = wgettext("Shaders");
 			Environment->addCheckBox(m_data->enable_shaders, rect, this,
-					GUI_ID_SHADERS_CB, wgettext("Shaders"));
+					GUI_ID_SHADERS_CB, text);
+			delete[] text;
 		}
 
 		{
 			core::rect<s32> rect(0, 0, option_w+20+20, 30);
 			rect += m_topleft_client + v2s32(option_x+175*2, option_y+20);
+			wchar_t* text = wgettext("Preload item visuals");
 			Environment->addCheckBox(m_data->preload_item_visuals, rect, this,
-					GUI_ID_PRELOAD_ITEM_VISUALS_CB, wgettext("Preload item visuals"));
+					GUI_ID_PRELOAD_ITEM_VISUALS_CB, text);
+			delete[] text;
 		}
 
 		{
 			core::rect<s32> rect(0, 0, option_w+20+20, 30);
 			rect += m_topleft_client + v2s32(option_x+175*2, option_y+20*2);
+			wchar_t* text = wgettext("Enable Particles");
 			Environment->addCheckBox(m_data->enable_particles, rect, this,
-					GUI_ID_ENABLE_PARTICLES_CB, wgettext("Enable Particles"));
+					GUI_ID_ENABLE_PARTICLES_CB, text);
+			delete[] text;
 		}
 
 		{
 			core::rect<s32> rect(0, 0, option_w+20+20, 30);
 			rect += m_topleft_client + v2s32(option_x+175*2, option_y+20*3);
+			wchar_t* text = wgettext("Finite liquid");
 			Environment->addCheckBox(m_data->liquid_finite, rect, this,
-					GUI_ID_LIQUID_FINITE_CB, wgettext("Finite liquid"));
+					GUI_ID_LIQUID_FINITE_CB, text);
+			delete[] text;
 		}
 
 		// Key change button
@@ -765,8 +882,10 @@ void GUIMainMenu::regenerateGui(v2u32 screensize)
 			/*rect += m_topleft_client + v2s32(m_size_client.X-120-30,
 					m_size_client.Y-30-20);*/
 			rect += m_topleft_client + v2s32(option_x, option_y+120);
+			wchar_t* text = wgettext("Change keys");
 			Environment->addButton(rect, this,
-					GUI_ID_CHANGE_KEYS_BUTTON, wgettext("Change keys"));
+					GUI_ID_CHANGE_KEYS_BUTTON, text);
+			delete[] text;
 		}
 		changeCtype("C");
 	}
@@ -822,7 +941,9 @@ void GUIMainMenu::drawMenu()
 	if (!skin)
 		return;
 	video::IVideoDriver* driver = Environment->getVideoDriver();
-	
+
+	/* Draw menu background */
+
 	/*video::SColor bgcolor(140,0,0,0);
 	driver->draw2DRectangle(bgcolor, AbsoluteRect, &AbsoluteClippingRect);*/
 
@@ -889,6 +1010,8 @@ void GUIMainMenu::drawMenu()
 		}
 	}
 
+	/* Draw UI elements */
+
 	gui::IGUIElement::draw();
 }
 
@@ -1013,8 +1136,13 @@ void GUIMainMenu::readInput(MainMenuData *dst)
 
 	{
 		gui::IGUIElement *e = getElementFromId(GUI_ID_WORLD_LISTBOX);
-		if(e != NULL && e->getType() == gui::EGUIET_LIST_BOX)
-			dst->selected_world = ((gui::IGUIListBox*)e)->getSelected();
+		if(e != NULL && e->getType() == gui::EGUIET_LIST_BOX){
+			int list_i = ((gui::IGUIListBox*)e)->getSelected();
+			if(list_i == -1)
+				dst->selected_world = -1;
+			else
+				dst->selected_world = m_world_indices[list_i];
+		}
 	}
 	{
 		ServerListSpec server =
@@ -1080,9 +1208,11 @@ bool GUIMainMenu::OnEvent(const SEvent& event)
 				readInput(&cur);
 				if (getTab() == TAB_MULTIPLAYER && cur.address == L"")
 				{
+					wchar_t* text = wgettext("Address required.");
 					(new GUIMessageMenu(env, parent, -1, menumgr,
-							wgettext("Address required."))
+							text)
 							)->drop();
+					delete[] text;
 					return true;
 				}
 				acceptInput();
@@ -1098,9 +1228,11 @@ bool GUIMainMenu::OnEvent(const SEvent& event)
 				MainMenuData cur;
 				readInput(&cur);
 				if(cur.selected_world == -1){
+					wchar_t* text = wgettext("Cannot delete world: Nothing selected");
 					(new GUIMessageMenu(env, parent, -1, menumgr,
-							wgettext("Cannot delete world: Nothing selected"))
+							text)
 							)->drop();
+					delete[] text;
 				} else {
 					WorldSpec spec = m_data->worlds[cur.selected_world];
 					// Get files and directories involved
@@ -1110,12 +1242,16 @@ bool GUIMainMenu::OnEvent(const SEvent& event)
 					// Launch confirmation dialog
 					ConfirmDestDeleteWorld *dest = new
 							ConfirmDestDeleteWorld(spec, this, paths);
-					std::wstring text = wgettext("Delete world");
+					wchar_t* text1 = wgettext("Delete world");
+					wchar_t* text2 = wgettext("Files to be deleted");
+					std::wstring text = text1;
 					text += L" \"";
 					text += narrow_to_wide(spec.name);
 					text += L"\"?\n\n";
-					text += wgettext("Files to be deleted");
+					text += text2;
 					text += L":\n";
+					delete[] text1;
+					delete[] text2;
 					for(u32 i=0; i<paths.size(); i++){
 						if(i == 3){ text += L"..."; break; }
 						text += narrow_to_wide(paths[i]) + L"\n";
@@ -1126,16 +1262,18 @@ bool GUIMainMenu::OnEvent(const SEvent& event)
 				return true;
 			}
 			case GUI_ID_CREATE_WORLD_BUTTON: {
-				std::vector<SubgameSpec> games = getAvailableGames();
+				const std::vector<SubgameSpec> &games = m_data->games;
 				if(games.size() == 0){
+					wchar_t* text = wgettext("Cannot create world: No games found");
 					GUIMessageMenu *menu = new GUIMessageMenu(env, parent,
 							-1, menumgr,
-							wgettext("Cannot create world: No games found"));
+							text);
 					menu->drop();
+					delete[] text;
 				} else {
 					CreateWorldDest *dest = new CreateWorldDestMainMenu(this);
 					GUICreateWorld *menu = new GUICreateWorld(env, parent, -1,
-							menumgr, dest, games);
+							menumgr, dest, games, m_data->selected_game);
 					menu->drop();
 				}
 				return true;
@@ -1145,9 +1283,11 @@ bool GUIMainMenu::OnEvent(const SEvent& event)
 				readInput(&cur);
 				if(cur.selected_world == -1)
 				{
+					wchar_t* text = wgettext("Cannot configure world: Nothing selected");
 					(new GUIMessageMenu(env, parent, -1, menumgr,
-							wgettext("Cannot configure world: Nothing selected"))
+							text)
 							)->drop();
+					delete[] text;
 				} 
 				else 
 				{
@@ -1180,8 +1320,12 @@ bool GUIMainMenu::OnEvent(const SEvent& event)
 				if (m_data->selected_serverlist == SERVERLIST_PUBLIC) // switch to favorite list
 				{
 					m_data->servers = ServerList::getLocal();
-					togglebutton->setText(wgettext("Show Public"));
-					title->setText(wgettext("Favorites:"));
+					wchar_t* text1 = wgettext("Show Public");
+					wchar_t* text2 = wgettext("Favorites:");
+					togglebutton->setText(text1);
+					title->setText(text2);
+					delete[] text1;
+					delete[] text2;
 					deletebutton->setVisible(true);
 					updateGuiServerList();
 					serverlist->setSelected(0);
@@ -1190,8 +1334,12 @@ bool GUIMainMenu::OnEvent(const SEvent& event)
 				else // switch to online list
 				{
 					m_data->servers = ServerList::getOnline();
-					togglebutton->setText(wgettext("Show Favorites"));
-					title->setText(wgettext("Public Server List:"));
+					wchar_t* text1 = wgettext("Show Favorites");
+					wchar_t* text2 = wgettext("Public Server List:");
+					togglebutton->setText(text1);
+					title->setText(text2);
+					delete[] text1;
+					delete[] text2;
 					deletebutton->setVisible(false);
 					updateGuiServerList();
 					serverlist->setSelected(0);
@@ -1201,17 +1349,42 @@ bool GUIMainMenu::OnEvent(const SEvent& event)
 			}
 			#endif
 			}
+			/* Game buttons */
+			int eid = event.GUIEvent.Caller->getID();
+			if(eid >= GUI_ID_GAME_BUTTON_FIRST &&
+					eid <= GUI_ID_GAME_BUTTON_MAX){
+				m_data->selected_game =
+						m_data->games[eid - GUI_ID_GAME_BUTTON_FIRST].id;
+				m_data->selected_game_name =
+						m_data->games[eid - GUI_ID_GAME_BUTTON_FIRST].name;
+				regenerateGui(m_screensize_old);
+			}
 		}
 		if(event.GUIEvent.EventType==gui::EGET_EDITBOX_ENTER)
 		{
 			switch(event.GUIEvent.Caller->getID())
 			{
 				case GUI_ID_ADDRESS_INPUT: case GUI_ID_PORT_INPUT: case GUI_ID_NAME_INPUT: case 264:
+				MainMenuData cur;
+				readInput(&cur);
+				if (getTab() == TAB_MULTIPLAYER && cur.address == L"")
+				{
+					wchar_t* text = wgettext("Address required.");
+					(new GUIMessageMenu(env, parent, -1, menumgr,
+							text)
+							)->drop();
+					delete[] text;
+					return true;
+				}
 				acceptInput();
 				quitMenu();
 				return true;
 			}
 		}
+		if(event.GUIEvent.EventType==gui::EGET_LISTBOX_CHANGED)
+		{
+			readInput(m_data);
+		}
 		if(event.GUIEvent.EventType==gui::EGET_LISTBOX_SELECTED_AGAIN)
 		{
 			switch(event.GUIEvent.Caller->getID())
@@ -1252,8 +1425,10 @@ void GUIMainMenu::deleteWorld(const std::vector<std::string> &paths)
 	// Delete files
 	bool did = fs::DeletePaths(paths);
 	if(!did){
+		wchar_t* text = wgettext("Failed to delete all world files");
 		GUIMessageMenu *menu = new GUIMessageMenu(env, parent,
-				-1, menumgr, wgettext("Failed to delete all world files"));
+				-1, menumgr, text);
+		delete[] text;
 		menu->drop();
 	}
 	// Quit menu to refresh it
