@@ -593,7 +593,7 @@ public:
 			{
 				tiledef[j] = f->tiledef[j];
 				if(tiledef[j].name == "")
-					tiledef[j].name = "unknown_block.png";
+					tiledef[j].name = "unknown_node.png";
 			}
 
 			bool is_liquid = false;
@@ -628,6 +628,10 @@ public:
 				f->solidness = 0;
 				f->visual_solidness = 1;
 				break;
+			case NDT_GLASSLIKE_FRAMED:
+				f->solidness = 0;
+				f->visual_solidness = 1;
+				break;
 			case NDT_ALLFACES:
 				f->solidness = 0;
 				f->visual_solidness = 1;
@@ -658,15 +662,11 @@ public:
 				break;
 			}
 
-			u8 material_type = 0;
-			if(is_liquid){
-				if(f->alpha == 255)
-					material_type = TILE_MATERIAL_LIQUID_OPAQUE;
-				else
-					material_type = TILE_MATERIAL_LIQUID_TRANSPARENT;
-			} else{
-				material_type = TILE_MATERIAL_BASIC;
-			}
+			u8 material_type;
+			if (is_liquid)
+				material_type = (f->alpha == 255) ? TILE_MATERIAL_LIQUID_OPAQUE : TILE_MATERIAL_LIQUID_TRANSPARENT;
+			else
+				material_type = (f->alpha == 255) ? TILE_MATERIAL_BASIC : TILE_MATERIAL_ALPHA;
 
 			// Tiles (fill in f->tiles[])
 			for(u16 j=0; j<6; j++){
